//-----------------------------------------------------------------------------
// File: mp_wrap.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#ifndef MAGIC_PARTICLES_DX
#define MAGIC_PARTICLES_DX

#ifdef _WINDOWS
#include <d3d11.h>
#include <DirectXMath.h>
#else
#include "pch.h"
#endif

#include "mp.h"
#include "magic.h"

using namespace DirectX;

#define SHADER_ALPHATEST_WRAP

#define MAX_SAMPLERS	32

class MP_Device_WRAP;

struct CONSTANT_BUFFER
{
	#ifdef MAGIC_3D
	MP_MATRIX Smat;
	#else
	XMFLOAT2 screen_scale;
	float v1,v2;
	#endif
};

// eng: Class storing the textural atlas
// rus: Класс, который хранит текстурный атлас
class MP_Atlas_WRAP : public MP_Atlas
{
protected:
	ID3D11Texture2D* texture;
	ID3D11ShaderResourceView* resource_view;

public:
	MP_Atlas_WRAP(int width, int height, const char* file);

	// eng: Destroy atlas texture
	// rus: Уничтожить текстуру атласа
	virtual void Destroy();

	// eng: Returns texture
	// rus: Возвращает текстуру
	ID3D11Texture2D* GetTexture(){return texture;}

	// eng: Loading of frame texture
	// rus: Загрузка текстурного кадра
	virtual void LoadTexture(const MAGIC_CHANGE_ATLAS* c);

	// eng: Cleaning up of rectangle
	// rus: Очистка прямоугольника
	virtual void CleanRectangle(const MAGIC_CHANGE_ATLAS* c);

	ID3D11ShaderResourceView* GetResourceView(){return resource_view;}
};

// ------------------------------------------------------------------------------

struct MP_VERTEX_SHADER_WRAP : public MP_VERTEX_SHADER
{
	ID3D11VertexShader* shader;
	ID3DBlob* shader_bytecode;

	MP_VERTEX_SHADER_WRAP();
	virtual ~MP_VERTEX_SHADER_WRAP();

	virtual void Generate(MAGIC_MATERIAL* m, MP_Device* device);
};

struct MP_PIXEL_SHADER_WRAP : public MP_PIXEL_SHADER
{
	ID3D11PixelShader* shader;

	MP_PIXEL_SHADER_WRAP();
	virtual ~MP_PIXEL_SHADER_WRAP();

	virtual void Generate(MAGIC_MATERIAL* m, MP_Device* device);
};

struct TEXTURE_STATES
{
	int sampler;
	bool changed_address;
	int u_address, v_address;
};


struct MP_LAYOUT_WRAP : public MP_LAYOUT
{
	ID3D11InputLayout* layout;

	MP_LAYOUT_WRAP();
	virtual ~MP_LAYOUT_WRAP();

	virtual void Generate(MAGIC_VERTEX_FORMAT* format, MP_Device* device);
};


struct MP_BUFFER_WRAP : public MP_BUFFER
{
protected:
	ID3D11Buffer* buffer;

public:
	MP_BUFFER_WRAP();

	virtual void Destroy();
	virtual void* Map(int  stride, MP_Device* device);
	virtual void Unmap(MP_Device* device);
};

// Компонент вершинного буфера
struct MP_VERTEX_BUFFER_WRAP : public MP_BUFFER_WRAP
{
public:
	MP_VERTEX_BUFFER_WRAP();

	virtual void Create(int new_length, MP_Device* device);
	virtual void Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device);
};

// Компонент индексного буфера
struct MP_INDEX_BUFFER_WRAP : public MP_BUFFER_WRAP
{
public:
	MP_INDEX_BUFFER_WRAP();

	virtual void Create(int new_length, MP_Device* device);
	virtual void Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device);
};

// ------------------------------------------------------------------------------

// eng: Class controlling drawing
// rus: Класс, который управляет рисованием
class MP_Device_WRAP : public MP_Device
{
protected:
	#ifndef EXT_DEVICE
	HWND hwnd;
	#endif

	ID3D11Device* device_3d;
	ID3D11DeviceContext* context_3d;
	
	ID3D11Buffer* constant_buffer;

	#ifndef EXT_DEVICE
	IDXGISwapChain* swap_chain;
	ID3D11RenderTargetView* target_view;
	#endif
	ID3D11RasterizerState* rasterizer_state;

	MP_MATRIX view_matrix;
	MP_MATRIX projection_matrix;

public:
	ID3D11BlendState* blend_state_normal;
	ID3D11BlendState* blend_state_add;
	ID3D11BlendState* blend_state_opacity;
	
	ID3D11SamplerState* sampler_states[4][4];
	int zbuffer_enabled;
	int zwrite_enabled;
	bool z_changed;

	ID3D11DepthStencilState* depth_states[2][2];

	TEXTURE_STATES texture_states[MAX_SAMPLERS];
	int k_changed_address;
	int m_changed_address[MAX_SAMPLERS];

public:
	#ifdef EXT_DEVICE
	MP_Device_WRAP(int width, int height, ID3D11Device* device_3d, ID3D11DeviceContext* context_3d);
	#else
	MP_Device_WRAP(int width, int height, HWND hwnd);
	#endif
	
	ID3D11Device* GetDevice3D(){return device_3d;}
	ID3D11DeviceContext* GetContext3D(){return context_3d;}

public:
	bool LoadTextureFromMemory(const char* data, int len, bool system_memory, ID3D11Texture2D** texture);
	bool LoadTextureFromFile(const char* texture_file, bool system_memory, ID3D11Texture2D** texture);

public:

	// eng: Creating
	// rus: Создание
	virtual bool Create();

	// eng: Destroying
	// rus: Уничтожение
	virtual void Destroy();

	#ifdef MAGIC_3D
	// eng: Setting of scene
	// rus: Настройка сцены
	virtual void SetScene3d(bool camera_mode);

	// eng: Camera position
	// rus: Положение камеры
	void SetCamera(MP_POSITION& eye, MP_POSITION& at, MP_POSITION& up);
	#endif

	// eng: Beginning of scene drawing
	// rus: Начало отрисовки сцены
	virtual void BeginScene();

	// eng: End of scene drawing
	// rus: Конец отрисовки сцены
	virtual void EndScene();

	// eng: Indicates that device is lost
	// rus: Проверка на потерю устройства рисования
	virtual bool IsLost();

	// rus: Установка формата вершин
	virtual void SetVertexFormat(MAGIC_RENDERING_START* start);

	// rus: Установка настроек визуализации
	virtual void SetRenderState(MAGIC_RENDER_STATE* state);

	// rus: Рисование порции вершин
	virtual void DrawVertices(int starting_index, int indexes_count, int max_vertices);

	void SetRenderMaterial(int material);

	// eng: Creating of vertex shader
	// rus: Создание вершинного шейдера
	virtual MP_ELEMENT* NewVertexShader();

	// eng: Creating of pixel shader
	// rus: Создание пиксельного шейдера
	virtual MP_ELEMENT* NewPixelShader();

	// eng: Creating of technique
	// rus: Создание техники
	virtual MP_ELEMENT* NewTechnique();

	// eng: Creating of layout
	// rus: Создание декларации вершин
	MP_LAYOUT* NewLayout();

	// eng: Creating of vertex buffer
	// rus: Создание вершинного буфера
	virtual MP_BUFFER* NewVertexBuffer();

	// eng: Creating of index buffer
	// rus: Создание индексного
	virtual MP_BUFFER* NewIndexBuffer();
};

#endif